/*
 * (C) Copyright 2017-2018 UCAR
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 */

#ifndef UFO_OPERATORS_GNSSRO_BNDNBAM_OBSGNSSROBNDNBAMTLAD_H_
#define UFO_OPERATORS_GNSSRO_BNDNBAM_OBSGNSSROBNDNBAMTLAD_H_

#include <memory>
#include <ostream>
#include <string>

#include "oops/base/Variables.h"
#include "oops/util/ObjectCounter.h"
#include "ufo/LinearObsOperatorBase.h"
#include "ufo/operators/gnssro/BndNBAM/ObsGnssroBndNBAM.h"
#include "ufo/operators/gnssro/BndNBAM/ObsGnssroBndNBAMTLAD.interface.h"

// Forward declarations
namespace ioda {
  class ObsSpace;
  class ObsVector;
}

namespace ufo {
  class GeoVaLs;
  class ObsDiagnostics;

// -----------------------------------------------------------------------------
// Gnssro BndNBAM observation operator tl/ad
//   -- to reproduce exactly the operational (2019) NBAM
class ObsGnssroBndNBAMTLAD : public LinearObsOperatorBase,
                          private util::ObjectCounter<ObsGnssroBndNBAMTLAD> {
 public:
  typedef GnssroBndNBAMParameters Parameters_;
  static const std::string classname() {return "ufo::ObsGnssroBndNBAMTLAD";}

  ObsGnssroBndNBAMTLAD(const ioda::ObsSpace &, const Parameters_ &);
  virtual ~ObsGnssroBndNBAMTLAD();

  // Obs Operators
  void setTrajectory(const GeoVaLs &, ObsDiagnostics &) override;
  void simulateObsTL(const GeoVaLs &, ioda::ObsVector &) const override;
  void simulateObsAD(GeoVaLs &, const ioda::ObsVector &) const override;

  // Other
  const oops::Variables & requiredVars() const override {return *varin_;}

  int & toFortran() {return keyOperGnssroBndNBAM_;}
  const int & toFortran() const {return keyOperGnssroBndNBAM_;}

 private:
  void print(std::ostream &) const override;
  F90hop keyOperGnssroBndNBAM_;
  std::unique_ptr<const oops::Variables> varin_;
};

// -----------------------------------------------------------------------------

}  // namespace ufo
#endif  // UFO_OPERATORS_GNSSRO_BNDNBAM_OBSGNSSROBNDNBAMTLAD_H_
